from motor import Motor
from time import sleep

## Parameters
# SPEED_MAX = 100
# SPEED_MIN = -100
SPEED_MAX = 70
SPEED_MIN = -70
DELTA_ACC = 10
DELTA_DEC = 10
FACTOR_TRN = 0.3
FACTOR_ROT = 0.5

class Esc:
    def __init__(self, pin_m1_ena, pin_m1_in1, pin_m1_in2,
                 pin_m2_ena, pin_m2_in1, pin_m2_in2):
        self.m1 = Motor("m1", pin_m1_ena, pin_m1_in1, pin_m1_in2)
        self.m2 = Motor("m2", pin_m2_ena, pin_m2_in1, pin_m2_in2)
        self.m1_speed = 0
        self.m2_speed = 0

    def left(self):
        if self.m2_speed > 0:
            self.m1_speed = self.m2_speed * FACTOR_TRN
        elif self.m2_speed < 0:
            self.m1_speed = self.m2_speed * FACTOR_TRN

    def right(self):
        if self.m1_speed > 0:
            self.m2_speed = self.m1_speed * FACTOR_TRN
        elif self.m1_speed < 0:
            self.m2_speed = self.m1_speed * FACTOR_TRN

    def accel(self):
        speed = max(self.m1_speed, self.m2_speed)
        self.m1_speed = min(speed + DELTA_ACC, SPEED_MAX)
        self.m2_speed = min(speed + DELTA_ACC, SPEED_MAX)

    def decel(self):
        speed = min(self.m1_speed, self.m2_speed)
        self.m1_speed = max(speed - DELTA_DEC, SPEED_MIN)
        self.m2_speed = max(speed - DELTA_DEC, SPEED_MIN)

    def slow(self):
        s1 = 0
        if self.m1_speed > DELTA_DEC:
            s1 = self.m1_speed - DELTA_DEC
        elif self.m1_speed < -DELTA_DEC:
            s1 = self.m1_speed + DELTA_DEC
        self.m1_speed = s1

        s2 = 0
        if self.m2_speed > DELTA_DEC:
            s2 = self.m2_speed - DELTA_DEC
        elif self.m2_speed < -DELTA_DEC:
            s2 = self.m2_speed + DELTA_DEC
        self.m2_speed = s2

    def stop(self):
        self.m1_speed = 0
        self.m2_speed = 0

    def rotate_left(self):
        self.m1_speed = SPEED_MIN * FACTOR_ROT
        self.m2_speed = SPEED_MAX * FACTOR_ROT

    def rotate_right(self):
        self.m1_speed = SPEED_MAX * FACTOR_ROT
        self.m2_speed = SPEED_MIN * FACTOR_ROT

    def apply(self):
        self.m1.set_speed(self.m1_speed)
        self.m2.set_speed(self.m2_speed)

    def get_status(self):
        return("{}, {}".format(self.m1.get_status(), self.m2.get_status()))
