# main.py
# standard
import time
# micropython
import ujson as json
import machine
import usocket as socket
# local
from esc import Esc
from credentials import SSID, PASS
import wifi
# import web

## Pins
PIN_M1_IN1 = 12 # D6
PIN_M1_IN2 = 13 # D7
PIN_M1_ENA = 15 # D8
PIN_M2_ENA = 5 # D1
PIN_M2_IN1 = 4 # D2
PIN_M2_IN2 = 0 # D3
## HOST
RX_ADDR = ("0.0.0.0", 8000)

## Parameters
FAIL_COUNT_MAX = 10

## Runtime
esc = None

def main():
    global esc
    # wifiSTA = wifi.WifiSTA(SSID, PASS)
    # wifiSTA.connect()
    wifiAP = wifi.WifiAP(SSID, PASS)
    wifiAP.connect()
    # Controller
    esc = Esc(PIN_M1_ENA, PIN_M1_IN1, PIN_M1_IN2,
              PIN_M2_ENA, PIN_M2_IN1, PIN_M2_IN2)
    # Socket
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    s.setsockopt(socket.SOL_SOCKET,socket.SO_REUSEADDR, 1)
    s.settimeout(0.1)
    s.bind(RX_ADDR)

    fail_count = 0
    values = {}
    while True:
        # wifiSTA.check()
        request = ""
        try:
            request, addr = s.recvfrom(256)
        except OSError as ex:
            pass
        except Exception as ex:
            print("Exception Type:{}, args: {}".format(type(ex).__name__, ex.args))

        ## read request
        if len(request) > 0:
            fail_count = 0
            req = request.decode()
            values = json.loads(req)
        else:
            fail_count += 1
            if fail_count > FAIL_COUNT_MAX:
                print("resetting values")
                fail_count = 0
                values = {}

        ## control
        # special
        if values.get("btn_b1") and not values.get("btn_b2"):
            esc.rotate_left()
        elif values.get("btn_b2") and not values.get("btn_b1"):
            esc.rotate_right()
        elif values.get("btn_b1") and values.get("btn_b2"):
            esc.stop()
        # accel or decel
        else:
            if values.get("btn_dn"):
                esc.decel()
            elif values.get("btn_up"):
                esc.accel()
            else:
                esc.slow()
            # left or right
            if values.get("btn_le"):
                esc.left()
            elif values.get("btn_ri"):
                esc.right()
        esc.apply()

        print("[{}] {}".format(fail_count, esc.get_status()))


if __name__ == "__main__":
    main()
